package perlchartdir;
require 5.004;

sub autoImport
{
	my $ver = shift;
	if (!(eval "
		package $ver;
		require DynaLoader;
		\@ISA = qw(DynaLoader);
		bootstrap $ver;
		sub perlchartdir::major_ver { return &major_ver; };
		sub perlchartdir::minor_ver { return &minor_ver; }
		sub perlchartdir::copyright { return &copyright; }
		sub perlchartdir::id { return &id; }
		sub perlchartdir::callMethod { return &callMethod; }
		") && ($@)) {
		die;
	}
}

if ($] >= 5.008) {
	use Config;
	if (($Config{"useithreads"}) || ($Config{"use5005threads"})) {
		if ($Config{"use64bitint"}) {
			autoImport("perlchartdir58i64mt");
		} else {
			autoImport("perlchartdir58mt");
		}
	} else {
		if ($Config{"use64bitint"}) {
			autoImport("perlchartdir58i64");
		} else {
			autoImport("perlchartdir58");
		}
	}
}
elsif ($] >= 5.006) {
	use Config;
	if (($Config{"useithreads"}) || ($Config{"use5005threads"})) {
		if ($Config{"use64bitint"}) {
			autoImport("perlchartdir56i64mt");
		} else {
			autoImport("perlchartdir56mt");
		}
	} else {
		if ($Config{"use64bitint"}) {
			autoImport("perlchartdir56i64");
		} else {
			autoImport("perlchartdir56");
		}
	}
}
elsif ($] >= 5.005) {
	autoImport("perlchartdir5005");
}
else {
	autoImport("perlchartdir5004");
}

my $dllVersion = (callMethod("getVersion") >> 16) & 0x7fff;
if (($dllVersion != 0x0300) && ($dllVersion != 0x0301))
{
	my $majorv = ($dllVersion >> 8) & 0xff;
	my $minorv = $dllVersion & 0xff;
	die "Version mismatch - \"perlchartdir.pm\" is of Ver 3.1, but \"chartdir.dll/libchartdir.so\" is of Ver $majorv.$minorv\n";
}

#///////////////////////////////////////////////////////////////////////////////////
#//	Internal functions
#///////////////////////////////////////////////////////////////////////////////////
sub checkarg
{
	my $params = shift;
	my $requiredNoOfParams = shift;
	if (!$requiredNoOfParams) { $requiredNoOfParams = 0; }
	my $actualNoOfParams = scalar(@$params);
	if ($actualNoOfParams > $requiredNoOfParams)
	{	die "Too many arguments; expecting $requiredNoOfParams but received $actualNoOfParams\n"; }
	my $noOfDefaultParams = scalar(@_);
	my $noOfMandatoryParams = $requiredNoOfParams - $noOfDefaultParams;
	if ($actualNoOfParams < $noOfMandatoryParams)
	{	die "Too few arguments; need at least $noOfMandatoryParams but received $actualNoOfParams\n"; }
	while ($actualNoOfParams < $requiredNoOfParams)
	{
		push(@$params, $_[$noOfDefaultParams + $actualNoOfParams - $requiredNoOfParams]);
		++$actualNoOfParams;
	}	
	return $params;
}

sub printerr
{
	my ($msg, $back) = @_;
	chop $msg;
	my @context = caller($back);
	if (substr($context[3], -2) eq "_r") {
		@context = caller($back + 2);
	}
	die "$msg when calling $context[3] at $context[1] line $context[2]\n"; 
}

sub func
{
	my $methodName = shift;
	my $ret = eval { 
		my $params = checkarg(@_);
		callMethod($methodName, @$params); 
		};
	if ((!$ret) && ($@)) { printerr($@, 2); }
	return $ret;	
}

sub method
{
	my $methodName = shift;
	my $params = shift;
	my $self = shift(@$params);
	my $ret = eval { 
		my $params = checkarg($params, @_);
		callMethod($methodName, $self->{"this"}, @$params); 
		};
	if ((!$ret) && ($@)) { printerr($@, 2); }
	return $ret;	
}

sub _r
{
	my ($args, $dargTable) = @_;
	
	my @back = caller(2);
	my $fullName = $back[3];
	$fullName =~ s/::/\./;
	my $methodName = $fullName;
	$methodName =~ s/.*\.//;
	my $params = $$dargTable{$methodName};
	my $ret;
	if ($params) {
		my @dargs = @$params;
		shift @dargs;
		my $ret = method($fullName, $args, @dargs);
		my $maker = $$params[0];
		if (defined($maker)) {
			return eval("new $maker('$ret')");
		} else {
			return $ret;
		}		
	} else {
		return method($fullName, $args, scalar(@$args) - 1);
	}
}

#///////////////////////////////////////////////////////////////////////////////////
#//	constants
#///////////////////////////////////////////////////////////////////////////////////

$BottomLeft = 1;
$BottomCenter = 2;
$BottomRight = 3;
$Left = 4;
$Center = 5;
$Right = 6;
$TopLeft = 7;
$TopCenter = 8;
$TopRight = 9;
$Top = $TopCenter;
$Bottom = $BottomCenter;

$Transparent = 0xff000000;
$Palette = 0xffff0000;
$BackgroundColor = 0xffff0000; 
$LineColor = 0xffff0001;
$TextColor = 0xffff0002;
$DataColor = 0xffff0008;
$SameAsMainColor = 0xffff0007;

$NoValue = +1.7e308;
$LogTick = 1.6E+308;
$TouchBar = -1.7E-100;

$NoAntiAlias = 0;
$AntiAlias = 1;
$AutoAntiAlias = 2;

$TryPalette = 0;
$ForcePalette = 1;
$NoPalette = 2;
$Quantize = 0;
$OrderedDither = 1;
$ErrorDiffusion = 2;

$BoxFilter = 0;
$LinearFilter = 1;
$QuadraticFilter = 2;
$BSplineFilter = 3;
$HermiteFilter = 4;
$CatromFilter = 5;
$MitchellFilter = 6;
$SincFilter = 7;
$LanczosFilter = 8;
$GaussianFilter = 9;
$HanningFilter = 10;
$HammingFilter = 11;
$BlackmanFilter = 12;
$BesselFilter = 13;

$PNG = 0;
$GIF = 1;
$JPG = 2;
$WMP = 3;
$BMP = 4;

$Overlay = 0;
$Stack = 1;
$Depth = 2;
$Side = 3;
$Percentage = 4;

$defaultPalette = [
	0xffffff, 0x000000, 0x000000, 0x808080, 
	0x808080, 0x808080, 0x808080, 0x808080,
	0xff3333, 0x33ff33, 0x6666ff, 0xffff00, 
	0xff66ff, 0x99ffff,	0xffcc33, 0xcccccc, 
	0xcc9999, 0x339966, 0x999900, 0xcc3300,	
	0x669999, 0x993333, 0x006600, 0x990099,
	0xff9966, 0x99ff99, 0x9999ff, 0xcc6600,
	0x33cc33, 0xcc99ff, 0xff6666, 0x99cc66,
	0x009999, 0xcc3333, 0x9933ff, 0xff0000,
	0x0000ff, 0x00ff00, 0xffcc99, 0x999999,
	-1
];
$whiteOnBlackPalette = [
	0x000000, 0xffffff, 0xffffff, 0x808080, 
	0x808080, 0x808080, 0x808080, 0x808080,
	0xff0000, 0x00ff00, 0x0000ff, 0xffff00, 
	0xff00ff, 0x66ffff,	0xffcc33, 0xcccccc, 
	0x9966ff, 0x339966, 0x999900, 0xcc3300,	
	0x99cccc, 0x006600, 0x660066, 0xcc9999,
	0xff9966, 0x99ff99, 0x9999ff, 0xcc6600,
	0x33cc33, 0xcc99ff, 0xff6666, 0x99cc66,
	0x009999, 0xcc3333, 0x9933ff, 0xff0000,
	0x0000ff, 0x00ff00, 0xffcc99, 0x999999,
	-1
];
$transparentPalette = [ 
	0xffffff, 0x000000, 0x000000, 0x808080, 
	0x808080, 0x808080, 0x808080, 0x808080,
	0x80ff0000, 0x8000ff00, 0x800000ff, 0x80ffff00, 
	0x80ff00ff, 0x8066ffff,	0x80ffcc33, 0x80cccccc, 
	0x809966ff, 0x80339966, 0x80999900, 0x80cc3300,
	0x8099cccc, 0x80006600, 0x80660066, 0x80cc9999,
	0x80ff9966, 0x8099ff99, 0x809999ff, 0x80cc6600,
	0x8033cc33, 0x80cc99ff, 0x80ff6666, 0x8099cc66,
	0x80009999, 0x80cc3333, 0x809933ff, 0x80ff0000,
	0x800000ff, 0x8000ff00, 0x80ffcc99, 0x80999999,
	-1
];

$NoSymbol = 0;
$SquareSymbol = 1;
$DiamondSymbol = 2;
$TriangleSymbol = 3;
$RightTriangleSymbol = 4;
$LeftTriangleSymbol = 5;
$InvertedTriangleSymbol = 6;
$CircleSymbol = 7;
$CrossSymbol = 8;
$Cross2Symbol = 9;

$DashLine = 0x0505;
$DotLine = 0x0202;
$DotDashLine = 0x05050205;
$AltDashLine = 0x0A050505;

$goldGradient = [0, 0xFFE743, 0x60, 0xFFFFE0, 0xB0, 0xFFF0B0, 0x100, 0xFFE743];
$silverGradient = [0, 0xC8C8C8, 0x60, 0xF8F8F8, 0xB0, 0xE0E0E0, 0x100, 0xC8C8C8];
$redMetalGradient = [0, 0xE09898, 0x60, 0xFFF0F0, 0xB0, 0xF0D8D8, 0x100, 0xE09898];
$blueMetalGradient = [0, 0x9898E0, 0x60, 0xF0F0FF, 0xB0, 0xD8D8F0, 0x100, 0x9898E0];
$greenMetalGradient = [0, 0x98E098, 0x60, 0xF0FFF0, 0xB0, 0xD8F0D8, 0x100, 0x98E098];

$NormalLegend = 0;
$ReverseLegend = 1;
$NoLegend = 2;

$SideLayout = 0;
$CircleLayout = 1;

$PixelScale = 0;
$XAxisScale = 1;
$YAxisScale = 2;
$AngularAxisScale = XAxisScale;
$RadialAxisScale = YAxisScale;

#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to libgraphics.h
#///////////////////////////////////////////////////////////////////////////////////

package TTFText;

#obsoleted constants - for compatibility only
$NoAntiAlias = $perlchartdir::NoAntiAlias;
$AntiAlias = $perlchartdir::AntiAlias;
$AutoAntiAlias = $perlchartdir::AutoAntiAlias;

%defaultArgs = ();

sub new
{
	my ($class, $this, $parent) = @_;
	my $self = {};
	$self->{"this"} = $this;
	$self->{"parent"} = $parent;
	return bless($self, $class);
}
sub DESTROY
{
	my $self = shift;
	perlchartdir::callMethod("TTFText.destroy", $self->{"this"})
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}	

sub getWidth { _r @_; }
sub getHeight { _r @_; }
sub getLineHeight { _r @_; }
sub getLineDistance { _r @_; }
sub draw { _r @_; }


package DrawArea;

#obsoleted constants - for compatibility only
$TryPalette = $perlchartdir::TryPalette;
$ForcePalette = $perlchartdir::ForcePalette;
$NoPalette = $perlchartdir::NoPalette;
$Quantize = $perlchartdir::Quantize;
$OrderedDither = $perlchartdir::OrderedDither;
$ErrorDiffusion = $perlchartdir::ErrorDiffusion;

%defaultArgs = (
	"setSize"=>[undef, 3, 0xffffff],
	"resize"=>[undef, 4, $perlchartdir::LinearFilter, 1],
	"move"=>[undef, 5, 0xffffff, $perlchartdir::LinearFilter, 1],
	"rotate"=>[undef, 6, 0xffffff, -1, -1, $perlchartdir::LinearFilter, 1],
	"line"=>[undef, 6, 1],
	"rect"=>[undef, 7, 0],
	"text2"=>[undef, 11, $perlchartdir::TopLeft],
	"rAffineTransform"=>[undef, 9, 0xffffff, $perlchartdir::LinearFilter, 1],
	"affineTransform"=>[undef, 9, 0xffffff, $perlchartdir::LinearFilter, 1],
	"sphereTransform"=>[undef, 5, 0xffffff, $perlchartdir::LinearFilter, 1],
	"hCylinderTransform"=>[undef, 4, 0xffffff, $perlchartdir::LinearFilter, 1],
	"vCylinderTransform"=>[undef, 4, 0xffffff, $perlchartdir::LinearFilter, 1],
	"vTriangleTransform"=>[undef, 4, -1, 0xffffff, $perlchartdir::LinearFilter, 1],
	"hTriangleTransform"=>[undef, 4, -1, 0xffffff, $perlchartdir::LinearFilter, 1],
	"shearTransform"=>[undef, 5, 0, 0xffffff, $perlchartdir::LinearFilter, 1],
	"waveTransform"=>[undef, 8, 0, 0, 0, 0xffffff, $perlchartdir::LinearFilter, 1],
	"outJPG"=>[undef, 2, 80],
	"outJPG2"=>[undef, 1, 80],
	"setAntiAlias"=>[undef, 2, 1, $perlchartdir::AutoAntiAlias],
	"dashLineColor"=>[undef, 2, $perlchartdir::DashLine],
	"patternColor2"=>[undef, 3, 0, 0],
	"gradientColor2"=>[undef, 5, 90, 1, 0, 0],
	"setDefaultFonts"=>[undef, 4, "", "", ""]
	);

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	if (not $this)
	{
		$self->{"this"} = perlchartdir::callMethod("DrawArea.create");
		$self->{"own_this"} = 1;
	}
	else
	{
		$self->{"this"} = $this;
		$self->{"own_this"} = 0;
	}
	return bless($self, $class);
}
sub DESTROY
{
	my $self = shift;
	if ($self->{"own_this"})
		{ perlchartdir::callMethod("DrawArea.destroy", $self->{"this"}); }
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}	

sub setSize { _r @_; }
sub resize { _r @_; }
sub getWidth { _r @_; }
sub getHeight { _r @_; }
sub setClipRect { _r @_; }
sub setBgColor { _r @_; }

sub move { _r @_; }
sub rotate { _r @_; }
sub hFlip() { _r @_; }
sub vFlip() { _r @_; }
sub clone
{
	my $ret = eval {
		my ($self, $d, $x, $y, $align, $newWidth, $newHeight, $ft, $blur) = @{perlchartdir::checkarg(\@_, 9,
			-1, -1, $perlchartdir::LinearFilter, 1)};
		perlchartdir::callMethod("DrawArea.clone", $self->{"this"}, $d->{"this"}, $x, $y, 
			$align, $newWidth, $newHeight, $ft, $blur);
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}

sub pixel { _r @_; }
sub getPixel { _r @_; }

sub hline { _r @_; }
sub vline { _r @_; }
sub line { _r @_; }
sub arc { _r @_; }

sub rect  { _r @_; }
sub polygon
{
	my $ret = eval {
		my ($self, $points, $edgeColor, $fillColor) = @{perlchartdir::checkarg(\@_, 4)};
		my @x = ();
		my @y = ();
		foreach my $p (@$points)
		{
			push(@x, $$p[0]);
			push(@y, $$p[1]);
		}
		perlchartdir::callMethod("DrawArea.polygon", $self->{"this"}, \@x, \@y, 
			$edgeColor, $fillColor);
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}
sub surface { _r @_; }
sub sector { _r @_; }
sub cylinder { _r @_; }
sub circle { _r @_; }

sub fill
{
	if (scalar(@_) > 4) { fill2(@_); }
	else { perlchartdir::method("DrawArea.fill", \@_, 3); }
}
sub fill2  { _r @_; }

sub text  { _r @_; }
sub text2 { _r @_; }
sub text3
{
	my $self = $_[0];
	return new TTFText(perlchartdir::method("DrawArea.text3", \@_, 3), $self);
}
sub text4
{
	my $self = $_[0];
	return new TTFText(perlchartdir::method("DrawArea.text4", \@_, 7), $self);
}

sub merge
{
	my $ret = eval {
		my ($self, $d, $x, $y, $align, $transparency) = @{perlchartdir::checkarg(\@_, 6)};
		perlchartdir::callMethod("DrawArea.merge", $self->{"this"}, $d->{"this"}, $x, $y, 
			$align, $transparency);
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}
sub tile
{
	my $ret = eval {
		my ($self, $d, $transparency) = @{perlchartdir::checkarg(\@_, 3)};
		perlchartdir::callMethod("DrawArea.tile", $self->{"this"}, $d->{"this"}, $transparency);
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}

sub setSearchPath { _r @_; }
sub loadGIF { _r @_; }
sub loadPNG { _r @_; }
sub loadJPG { _r @_; }
sub loadWMP { _r @_; }
sub load { _r @_; }

sub rAffineTransform { _r @_; }
sub affineTransform { _r @_; }
sub sphereTransform { _r @_; }
sub hCylinderTransform { _r @_; }
sub vCylinderTransform { _r @_; }
sub vTriangleTransform { _r @_; }
sub hTriangleTransform { _r @_; }
sub shearTransform { _r @_; }
sub waveTransform { _r @_; }

sub out { _r @_; }
sub outGIF { _r @_; }
sub outPNG { _r @_; }
sub outJPG { _r @_; }
sub outWMP { _r @_; }
sub outBMP { _r @_; }
sub outGIF2 { _r @_; }
sub outPNG2 { _r @_; }
sub outJPG2 { _r @_; }
sub outWMP2 { _r @_; }
sub outBMP2 { _r @_; }

sub setPaletteMode { _r @_; }
sub setDitherMethod { _r @_; }
sub setTransparentColor { _r @_; }
sub setAntiAliasText { _r @_; }
sub setAntiAlias { _r @_; }
sub setInterlace { _r @_; }

sub setColorTable { _r @_; }
sub getARGBColor { _r @_; }
sub halfColor { _r @_; }

sub dashLineColor { _r @_; }
sub patternColor
{
	if (scalar(@_) < 3 or not ref($_[1])) { return patternColor2(@_); }
	return perlchartdir::method("DrawArea.patternColor", \@_, 4, 0, 0);
}
sub patternColor2 { _r @_; }
sub gradientColor
{
	if (scalar(@_) < 7) { return gradientColor2(@_); }
	return perlchartdir::method("DrawArea.gradientColor", \@_, 6);
}
sub gradientColor2 { _r @_; }

sub setDefaultFonts { _r @_; }
sub setFontTable { _r @_; }


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to drawobj.h
#///////////////////////////////////////////////////////////////////////////////////
package Box;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setPos
{
	perlchartdir::method("Box.setPos", \@_, 2);
}
sub setSize
{
	perlchartdir::method("Box.setSize", \@_, 2);
}
sub getWidth
{
	perlchartdir::method("Box.getWidth", \@_, 0);
}
sub getHeight
{
	perlchartdir::method("Box.getHeight", \@_, 0);
}
sub setBackground
{
	perlchartdir::method("Box.setBackground", \@_, 3, -1, 0);
}	
sub getImageCoor
{
	return perlchartdir::method("Box.getImageCoor", \@_, 2, 0, 0);
}

package TextBox;
@ISA = ("Box");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setText
{
	perlchartdir::method("TextBox.setText", \@_, 1);
}
sub setFontStyle
{
	perlchartdir::method("TextBox.setFontStyle", \@_, 2, 0);
}
sub setFontSize
{
	perlchartdir::method("TextBox.setFontSize", \@_, 2, 0);
}
sub setFontAngle
{
	perlchartdir::method("TextBox.setFontAngle", \@_, 2, 0);
}
sub setFontColor
{
	perlchartdir::method("TextBox.setFontColor", \@_, 1);
}
sub setMargin2
{
	perlchartdir::method("TextBox.setMargin2", \@_, 4);
}
sub setMargin
{
	perlchartdir::method("TextBox.setMargin", \@_, 1);
}
sub setAlignment
{
	perlchartdir::method("TextBox.setAlignment", \@_, 1);
}


package Line;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub setPos
{
	perlchartdir::method("Line.setPos", \@_, 4);
}
sub setColor
{
	perlchartdir::method("Line.setColor", \@_, 1);
}
sub setWidth
{
	perlchartdir::method("Line.setWidth", \@_, 1);
}

#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to basechart.h
#///////////////////////////////////////////////////////////////////////////////////
package LegendBox;
@ISA = ("TextBox");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub addKey
{
	my $ret = eval {
		my ($self, $text, $color, $lineWidth, $drawarea) = @{perlchartdir::checkarg(\@_, 5, 0, '$$pointer$$null')};
		if (ref($drawarea)) { $drawarea = $drawarea->{"this"}; }
		perlchartdir::callMethod("LegendBox.addKey", $self->{"this"}, $text, $color, $lineWidth, $drawarea);
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}
sub setKeySize
{
	perlchartdir::method("LegendBox.setKeySize", \@_, 3, -1, -1);
}
sub setKeySpacing
{
	perlchartdir::method("LegendBox.setKeySpacing", \@_, 2, -1);
}
sub getImageCoor2
{
	return perlchartdir::method("LegendBox.getImageCoor", \@_, 3, 0, 0);
}
sub getHTMLImageMap
{
	return perlchartdir::method("LegendBox.getHTMLImageMap", \@_, 5, "", "", 0, 0);
}


package BaseChart;

#obsoleted - for compatibility only.
$PNG = $perlchartdir::PNG;
$GIF = $perlchartdir::GIF;
$JPG = $perlchartdir::JPG;
$WMP = $perlchartdir::WMP;

sub DESTROY
{
	my $self = shift;
	perlchartdir::callMethod("BaseChart.destroy", $self->{"this"})
}
	
#//////////////////////////////////////////////////////////////////////////////////////
#//	set overall chart
#//////////////////////////////////////////////////////////////////////////////////////
sub setSize
{
	perlchartdir::method("BaseChart.setSize", \@_, 2);
}
sub setBorder
{
	perlchartdir::method("BaseChart.setBorder", \@_, 1);
}
sub setBackground
{
	perlchartdir::method("BaseChart.setBackground", \@_, 3, -1, 0);
}
sub setWallpaper
{
	perlchartdir::method("BaseChart.setWallpaper", \@_, 1);
}
sub setBgImage
{
	perlchartdir::method("BaseChart.setBgImage", \@_, 2, $perlchartdir::Center);
}
sub setTransparentColor
{
	perlchartdir::method("BaseChart.setTransparentColor", \@_, 1);
}
sub setAntiAlias
{
	perlchartdir::method("BaseChart.setAntiAlias", \@_, 2, 1, $perlchartdir::AutoAntiAlias);
}
sub setSearchPath
{
	perlchartdir::method("BaseChart.setSearchPath", \@_, 1);
}
sub addTitle2
{
	return new TextBox(perlchartdir::method("BaseChart.addTitle2", \@_, 7, "", 12, 
		$perlchartdir::TextColor, $perlchartdir::Transparent, $perlchartdir::Transparent));
}
sub addTitle
{
	return new TextBox(perlchartdir::method("BaseChart.addTitle", \@_, 6, "", 12, 
		$perlchartdir::TextColor, $perlchartdir::Transparent, $perlchartdir::Transparent));
}
sub addLegend
{
	return new LegendBox(perlchartdir::method("BaseChart.addLegend", \@_, 5, 1, "", 10));
}
sub getLegend
{
	return new LegendBox(perlchartdir::method("BaseChart.getLegend", \@_));
}
#//////////////////////////////////////////////////////////////////////////////////////
#//	drawing primitives
#//////////////////////////////////////////////////////////////////////////////////////
sub getDrawArea
{
	return new DrawArea(perlchartdir::method("BaseChart.getDrawArea", \@_));
}
sub addDrawObj
{
	my $ret = eval {
		my ($self, $obj) = @{perlchartdir::checkarg(\@_, 2)};
		perlchartdir::callMethod("BaseChart.addDrawObj", $self->{"this"}, $obj->{"this"});
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
	return $obj;
}
sub addText
{
	return new TextBox(perlchartdir::method("BaseChart.addText", \@_, 9, "", 8, 
		$perlchartdir::TextColor, $perlchartdir::TopLeft, 0, 0));
}
sub addLine
{
	return new Line(perlchartdir::method("BaseChart.addLine", \@_, 6, $perlchartdir::LineColor, 1));
}
#//////////////////////////////////////////////////////////////////////////////////////
#//	color management methods
#//////////////////////////////////////////////////////////////////////////////////////
sub setColor
{
	perlchartdir::method("BaseChart.setColor", \@_, 2);
}
sub setColors
{
	perlchartdir::method("BaseChart.setColors", \@_, 1);
}	
sub setColors2
{
	perlchartdir::method("BaseChart.setColors2", \@_, 2);
}
sub getColor
{
	return perlchartdir::method("BaseChart.getColor", \@_, 1);
}
sub halfColor
{
	return perlchartdir::method("BaseChart.halfColor", \@_, 1);
}
sub autoColor
{
	return perlchartdir::method("BaseChart.autoColor", \@_);
}
sub dashLineColor
{
	return perlchartdir::method("BaseChart.dashLineColor", \@_, 2);
}
sub patternColor
{
	if (scalar(@_) < 3 or not ref($_[1])) { return patternColor2(@_); }
	return perlchartdir::method("BaseChart.patternColor", \@_, 4, 0, 0);
}
sub patternColor2
{
	return perlchartdir::method("BaseChart.patternColor2", \@_, 3, 0, 0);
}
sub gradientColor
{
	if (scalar(@_) < 7) { return gradientColor2(@_); }
	return perlchartdir::method("BaseChart.gradientColor", \@_, 6);
}
sub gradientColor2
{
	return perlchartdir::method("BaseChart.gradientColor2", \@_, 5, 90, 1, 0, 0);
}
#/////////////////////////////////////////////////////////////////////////////////////
#//	locale support
#//////////////////////////////////////////////////////////////////////////////////////
sub setDefaultFonts
{
	perlchartdir::method("BaseChart.setDefaultFonts", \@_, 4, "", "", "");
}
sub setFontTable
{
	perlchartdir::method("BaseChart.setFontTable", \@_, 2);
}
sub setNumberFormat
{
	perlchartdir::method("BaseChart.setNumberFormat", \@_, 3, "~", ".", "-");
}
sub setMonthNames
{
	perlchartdir::method("BaseChart.setMonthNames", \@_, 1);
}
sub setWeekDayNames
{
	perlchartdir::method("BaseChart.setWeekDayNames", \@_, 1);
}
sub setAMPM
{
	perlchartdir::method("BaseChart.setAMPM", \@_, 2);
}		
#//////////////////////////////////////////////////////////////////////////////////////
#//	chart creation methods
#//////////////////////////////////////////////////////////////////////////////////////
sub layout
{
	perlchartdir::method("BaseChart.layout", \@_);
}
sub makeChart
{
	return perlchartdir::method("BaseChart.makeChart", \@_, 1);
}
sub makeChart2
{
	return perlchartdir::method("BaseChart.makeChart2", \@_, 1);
}
sub makeChart3
{
	return new DrawArea(perlchartdir::method("BaseChart.makeChart3", \@_, 0));
}
sub makeTmpFile
{
	my $self = shift;
	my $path = shift;
	my $imageFormat = (shift or $perlchartdir::PNG);
	my $lifeTime = (shift or 600);
	
	if ($imageFormat == $perlchartdir::JPG) { $imageFormat = "jpg"; }
	elsif ($imageFormat == $perlchartdir::GIF) { $imageFormat = "gif"; }
	elsif ($imageFormat == $perlchartdir::BMP) { $imageFormat = "bmp"; }
	elsif ($imageFormat == $perlchartdir::WMP) { $imageFormat = "wbmp"; }
	else { $imageFormat = "png"; }
	
	my $filename = perlchartdir::tmpFile2($path, $lifeTime) . "." . $imageFormat;
	if ($self->makeChart($path . "/" . $filename)) {
		return $filename;
	}
	else {
		return "";
	}
}
sub getHTMLImageMap
{
	return perlchartdir::method("BaseChart.getHTMLImageMap", \@_, 5, "", "", 0, 0);
}


package MultiChart;
@ISA = ("BaseChart");

sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("MultiChart.create", \@_, 5, 
		$perlchartdir::BackgroundColor, $perlchartdir::Transparent, 0);
	$self->{"dependencies"} = [];
	return bless($self, $class);
}
sub addChart
{
	my $ret = eval {
		my ($self, $x, $y, $c) = @{perlchartdir::checkarg(\@_, 4)};
		perlchartdir::callMethod("MultiChart.addChart", $self->{"this"}, $x, $y, $c->{"this"});
		push @{$self->{"dependencies"}}, $c;
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }		
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to piechart.h
#///////////////////////////////////////////////////////////////////////////////////
package Sector;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub setExplode
{
	perlchartdir::method("Sector.setExplode", \@_, 1, -1);
}
sub setLabelFormat
{
	perlchartdir::method("Sector.setLabelFormat", \@_, 1);
}
sub setLabelStyle
{
	return new TextBox(perlchartdir::method("Sector.setLabelStyle", \@_, 3, "", 8, $perlchartdir::TextColor));
}
sub setLabelLayout
{
	perlchartdir::method("Sector.setLabelLayout", \@_, 2, -1);
}
sub setLabelPos
{
	perlchartdir::method("Sector.setLabelPos", \@_, 2, -1);
}
sub setJoinLine
{
	perlchartdir::method("Sector.setJoinLine", \@_, 2, 1);
}	
sub setColor
{ 
	perlchartdir::method("Sector.setColor", \@_, 3, -1. -1);
}
sub getImageCoor
{
	return perlchartdir::method("Sector.getImageCoor", \@_, 2, 0, 0);
}
sub getLabelCoor
{
	return perlchartdir::method("Sector.getLabelCoor", \@_, 2, 0, 0);
}


package PieChart;
@ISA = ("BaseChart");

sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("PieChart.create", \@_, 5, 
		$perlchartdir::BackgroundColor, $perlchartdir::Transparent, 0);
	return bless($self, $class);
}
sub setPieSize
{
	perlchartdir::method("PieChart.setPieSize", \@_, 3);
}
sub set3D
{
	if (ref($_[1])) { set3D2(@_); }
	else { perlchartdir::method("PieChart.set3D", \@_, 3, -1, -1, 0); }
}
sub set3D2
{
	perlchartdir::method("PieChart.set3D2", \@_, 3, 45, 0);
}
sub setStartAngle
{
	perlchartdir::method("PieChart.setStartAngle", \@_, 2, 1);
}
sub setExplode
{
	perlchartdir::method("PieChart.setExplode", \@_, 2, -1);
}
sub setExplodeGroup
{
	perlchartdir::method("PieChart.setExplodeGroup", \@_, 3, -1);
}
sub setLabelFormat
{
	perlchartdir::method("PieChart.setLabelFormat", \@_, 1);
}
sub setLabelStyle
{
	return new TextBox(perlchartdir::method("PieChart.setLabelStyle", \@_, 3, "", 8, $perlchartdir::TextColor));
}
sub setLabelLayout
{
	perlchartdir::method("PieChart.setLabelLayout", \@_, 4, -1, -1, -1);
}
sub setLabelPos
{
	perlchartdir::method("PieChart.setLabelPos", \@_, 2, -1);	
}
sub setJoinLine
{
	perlchartdir::method("PieChart.setJoinLine", \@_, 2, 1);
}
sub setLineColor
{
	perlchartdir::method("PieChart.setLineColor", \@_, 2, -1);
}
sub setData
{
	perlchartdir::method("PieChart.setData", \@_, 2, []);
}
sub addExtraField
{
	perlchartdir::method("PieChart.addExtraField", \@_, 1);
}
sub sector
{
	return new Sector(perlchartdir::method("PieChart.sector", \@_, 1));
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to axis.h
#///////////////////////////////////////////////////////////////////////////////////
package Mark;
@ISA = ("TextBox");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub setValue
{
	perlchartdir::method("Mark.setValue", \@_, 1);
}
sub setMarkColor
{
	perlchartdir::method("Mark.setMarkColor", \@_, 3, -1, -1);
}
sub setLineWidth
{
	perlchartdir::method("Mark.setLineWidth", \@_, 1);
}
sub setDrawOnTop
{
	perlchartdir::method("Mark.setDrawOnTop", \@_, 1);
}
sub getLine
{
	return perlchartdir::method("Mark.getLine", \@_, 0);
}


package Axis;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub setLabelStyle
{
	return new TextBox(perlchartdir::method("Axis.setLabelStyle", \@_, 4, "",
		8, $perlchartdir::TextColor, 0));
}
sub setLabelFormat
{
	perlchartdir::method("Axis.setLabelFormat", \@_, 1);
}
sub setLabelGap
{
	perlchartdir::method("Axis.setLabelGap", \@_, 1);
}
sub setTitle
{
	return new TextBox(perlchartdir::method("Axis.setTitle", \@_, 4, 
		"arialbd.ttf", 8, $perlchartdir::TextColor));
}
sub setTitlePos
{
	perlchartdir::method("Axis.setTitlePos", \@_, 2, 6);
}
sub setColors
{
	perlchartdir::method("Axis.setColors", \@_, 4, $perlchartdir::TextColor, -1, -1);
}
sub setTickLength
{
	if (scalar(@_) == 3) { setTickLength2(@_); }
	else { perlchartdir::method("Axis.setTickLength", \@_, 1); }
}
sub setTickLength2
{
	perlchartdir::method("Axis.setTickLength2", \@_, 2);
}
sub setTickWidth
{
	perlchartdir::method("Axis.setTickWidth", \@_, 2, -1);
}
sub setTickColor
{
	perlchartdir::method("Axis.setTickColor", \@_, 2, -1);
}

sub setWidth
{
	perlchartdir::method("Axis.setWidth", \@_, 1);
}
sub setLength
{
	perlchartdir::method("Axis.setLength", \@_, 1);
}
sub setPos
{
	perlchartdir::method("Axis.setPos", \@_, 3, $perlchartdir::Center);
}
sub setTopMargin
{
	setMargin(@_);
}
sub setMargin
{
	perlchartdir::method("Axis.setMargin", \@_, 2, 0);
}
sub setIndent
{
	perlchartdir::method("Axis.setIndent", \@_, 1);
}

sub setAutoScale
{
	perlchartdir::method("Axis.setAutoScale", \@_, 3, 0, 0, 0.8);
}
sub setRounding
{
	perlchartdir::method("Axis.setRounding", \@_, 2);
}
sub setTickDensity
{
	perlchartdir::method("Axis.setTickDensity", \@_, 2, 20);
}
sub setReverse
{
	perlchartdir::method("Axis.setReverse", \@_, 1, 1);
}

sub setLabels
{
	if (scalar(@_) < 3) {
		return new TextBox(perlchartdir::method("Axis.setLabels", \@_, 1));
	} else {
		return setLabels2(@_);	
	}
}
sub setLabels2
{
	return new TextBox(perlchartdir::method("Axis.setLabels2", \@_, 2, ""));
}
sub setLinearScale
{
	if (scalar(@_) < 3) {
		setLinearScale3(@_);
	} 
	elsif (ref($_[3])) {
		setLinearScale2(@_);
	}
	else {
		perlchartdir::method("Axis.setLinearScale", \@_, 4, 0, 0);
	}
}
sub setLinearScale2
{
	perlchartdir::method("Axis.setLinearScale2", \@_, 3);
}
sub setLinearScale3
{
	perlchartdir::method("Axis.setLinearScale3", \@_, 1, "");
}
sub setLogScale
{
	if (scalar(@_) < 3) {
		setLogScale3(@_);
	}
	elsif (ref($_[3])) {
		setLogScale2(@_);
	}
	else {
		perlchartdir::method("Axis.setLogScale", \@_, 4, 0, 0);
	}
}
sub setLogScale2
{
	if (ref($_[3])) {
		perlchartdir::method("Axis.setLogScale2", \@_, 3);
	} else {
		#compatibility with ChartDirector Ver 2.5
		setLogScale(@_);
	}	
}
sub setLogScale3
{
	if (($_[1]) && ($_[1] =~ m/^-?\d+$/)) {
		#compatibility with ChartDirector Ver 2.5
		if ($_[1]) { setLogScale3($_[0]); }
		else { setLinearScale3($_[0]); }
	} else {
		perlchartdir::method("Axis.setLogScale3", \@_, 1, "");
	}
}
sub setDateScale
{
	if (scalar(@_) < 3) {
		setDateScale3(@_);
	}
	elsif (ref($_[3])) {
		setDateScale2(@_);
	}
	else {
		perlchartdir::method("Axis.setDateScale", \@_, 4, 0, 0);
	}
}
sub setDateScale2
{
	perlchartdir::method("Axis.setDateScale2", \@_, 3);
}
sub setDateScale3
{
	perlchartdir::method("Axis.setDateScale3", \@_, 1, "");
}

sub addLabel
{
	perlchartdir::method("Axis.addLabel", \@_, 2);
}
sub addMark
{
	return new Mark(perlchartdir::method("Axis.addMark", \@_, 5, "", "", 8));
}
sub addZone
{
	perlchartdir::method("Axis.addZone", \@_, 3);
}

sub getCoor
{
	return perlchartdir::method("Axis.getCoor", \@_, 1);
}
sub getLength
{
	return perlchartdir::method("Axis.getLength", \@_);
}
sub getMinValue
{
	return perlchartdir::method("Axis.getMinValue", \@_);
}
sub getMaxValue
{
	return perlchartdir::method("Axis.getMaxValue", \@_);
}
sub getScaleType
{
	return perlchartdir::method("Axis.getScaleType", \@_);
}

sub getTicks
{
	return perlchartdir::method("Axis.getTicks", \@_);
}
sub getLabel
{
	return perlchartdir::method("Axis.getLabel", \@_, 1);
}


package AngularAxis;

%defaultArgs = (
	"setLabelStyle"=>["TextBox", 4, "bold", 10, $perlchartdir::TextColor, 0],
	"setReverse"=>[undef, 1, 1],
	"setLabels2"=>["TextBox", 2, ""]
	);
	
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}	
sub setLabelStyle { _r(@_); }
sub setLabelGap { _r(@_); }
sub setLabels
{
	if (scalar(@_) < 3) {
		return new TextBox(perlchartdir::method("AngularAxis.setLabels", \@_, 1));
	} else {
		return setLabels2(@_);
	}
}
sub setLabels2 { _r(@_); }
sub addLabel { _r(@_); }
sub setLinearScale
{
	if (ref($_[3])) {
		setLinearScale2(@_);
	} else {
		perlchartdir::method("AngularAxis.setLinearScale", \@_, 4, 0, 0);
	}
}
sub setLinearScale2 { _r(@_); }
sub getCoor { _r(@_); }
sub getTicks { _r(@_); }
sub getLabel { _r(@_); }


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to layer.h
#///////////////////////////////////////////////////////////////////////////////////
package DataSet;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
} 
sub setData
{
	perlchartdir::method("DataSet.setData", \@_, 1);
}
sub setDataName
{
	perlchartdir::method("DataSet.setDataName", \@_, 1);
}
sub setDataColor
{
	perlchartdir::method("DataSet.setDataColor", \@_, 4, -1, -1, -1);
}
sub setUseYAxis2
{
	perlchartdir::method("DataSet.setUseYAxis2", \@_, 1, 1);
}
sub setLineWidth
{
	perlchartdir::method("DataSet.setLineWidth", \@_, 1);
}
sub setDataLabelFormat
{
	perlchartdir::method("DataSet.setDataLabelFormat", \@_, 1);
}
sub setDataLabelStyle
{
	return new TextBox(perlchartdir::method("DataSet.setDataLabelStyle", \@_, 4, "", 8, $perlchartdir::TextColor, 0));
}
sub setDataSymbol
{
	if ((scalar(@_) < 3) && ($_[1]) && ($_[1] !~ m/^\d+$/)) {
		setDataSymbol2(@_);
	} else {
		perlchartdir::method("DataSet.setDataSymbol", \@_, 5, 5, -1, -1, 1);
	}
}
sub setDataSymbol2
{
	if (ref($_[1])) {
		setDataSymbol3(@_);
	} else {
		perlchartdir::method("DataSet.setDataSymbol2", \@_, 1);
	}
}
sub setDataSymbol3
{
	my $ret = eval {
		my ($self, $d) = @{perlchartdir::checkarg(\@_, 2)};
		perlchartdir::callMethod("DataSet.setDataSymbol3", $self->{"this"}, $d->{"this"});
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}

package Layer;

#obsoleted - for compatibility only
$Overlay = $perlchartdir::Overlay;
$Stack = $perlchartdir::Stack;
$Depth = $perlchartdir::Depth;
$Side = $perlchartdir::Side;

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setSize
{
	perlchartdir::method("Layer.setSize", \@_, 5, 0);
}
sub setBorderColor
{
	perlchartdir::method("Layer.setBorderColor", \@_, 2, 0);
}
sub set3D
{
	perlchartdir::method("Layer.set3D", \@_, 2, -1, 0);
}
sub set3D2
{
	perlchartdir::method("Layer.set3D2", \@_, 4);
}
sub setLineWidth
{
	perlchartdir::method("Layer.setLineWidth", \@_, 1);
}
sub setDataCombineMethod
{
	perlchartdir::method("Layer.setDataCombineMethod", \@_, 1);
}
sub setLegend
{
	perlchartdir::method("Layer.setLegend", \@_, 1);
}
sub addDataSet
{
	return new DataSet(perlchartdir::method("Layer.addDataSet", \@_, 3, -1, ""));
}
sub addDataGroup
{
	perlchartdir::method("Layer.addDataGroup", \@_, 1, "");
}
sub addExtraField
{
	perlchartdir::method("Layer.addExtraField", \@_, 1);
}
sub getDataSet
{
	return new DataSet(perlchartdir::method("Layer.getDataSet", \@_, 1));
}
sub setUseYAxis2
{
	perlchartdir::method("Layer.setUseYAxis2", \@_, 1, 1);
}
sub setXData
{
	if (scalar(@_) > 2) { setXData2(@_); }
	else { perlchartdir::method("Layer.setXData", \@_, 1); }
}
sub setXData2
{
	perlchartdir::method("Layer.setXData2", \@_, 2);
}
sub getMinX
{
	return perlchartdir::method("Layer.getMinX", \@_);
}
sub getMaxX
{
	return perlchartdir::method("Layer.getMaxX", \@_);
}
sub getMaxY
{
	return perlchartdir::method("Layer.getMaxY", \@_, 1, 1);
}
sub getMinY
{
	return perlchartdir::method("Layer.getMinY", \@_, 1, 1);
}
sub getDepthX
{
	return perlchartdir::method("Layer.getDepthX", \@_);
}
sub getDepthY
{
	return perlchartdir::method("Layer.getDepthY", \@_);
}
sub getXCoor
{
	return perlchartdir::method("Layer.getXCoor", \@_, 1);
}
sub getYCoor
{
	return perlchartdir::method("Layer.getYCoor", \@_, 2, 1);
}
sub xZoneColor
{
	return perlchartdir::method("Layer.xZoneColor", \@_, 3);
}
sub yZoneColor
{
	return perlchartdir::method("Layer.yZoneColor", \@_, 4, 1);
}
sub setDataLabelFormat
{
	perlchartdir::method("Layer.setDataLabelFormat", \@_, 1);
}
sub setDataLabelStyle
{
	return new TextBox(perlchartdir::method("Layer.setDataLabelStyle", \@_, 4, "", 8, $perlchartdir::TextColor, 0));
}
sub setAggregateLabelFormat
{
	perlchartdir::method("Layer.setAggregateLabelFormat", \@_, 1);
}
sub setAggregateLabelStyle
{
	return new TextBox(perlchartdir::method("Layer.setAggregateLabelStyle", \@_, 4, "", 8, $perlchartdir::TextColor, 0));
}
sub addCustomDataLabel
{
	return new TextBox(perlchartdir::method("Layer.addCustomDataLabel", \@_, 7, "", 8, $perlchartdir::TextColor, 0));
}
sub addCustomAggregateLabel
{
	return new TextBox(perlchartdir::method("Layer.addCustomAggregateLabel", \@_, 6, "", 8, $perlchartdir::TextColor, 0));
}
sub getImageCoor
{
	if (scalar(@_) == 2) { return getImageCoor2(@_); }
	return perlchartdir::method("Layer.getImageCoor", \@_, 4, 0, 0);
}
sub getImageCoor2
{
	return perlchartdir::method("Layer.getImageCoor2", \@_, 3, 0, 0);
}
sub getHTMLImageMap
{
	return perlchartdir::method("Layer.getHTMLImageMap", \@_, 5, "", "", 0, 0);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to barlayer.h
#///////////////////////////////////////////////////////////////////////////////////
package BarLayer;
@ISA = ("Layer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setBarGap
{
	perlchartdir::method("BarLayer.setBarGap", \@_, 2, 0.2);
}
sub setBarWidth
{
	perlchartdir::method("BarLayer.setBarWidth", \@_, 2, -1);
}
sub setMinLabelSize
{
	perlchartdir::method("BarLayer.setMinLabelSize", \@_, 1);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to linelayer.h
#///////////////////////////////////////////////////////////////////////////////////
package LineLayer;
@ISA = ("Layer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setGapColor
{
	perlchartdir::method("LineLayer.setGapColor", \@_, 2, -1);
}
sub setImageMapWidth
{
	perlchartdir::method("LineLayer.setImageMapWidth", \@_, 1);
}
sub setSymbolScale
{
	perlchartdir::method("LineLayer.setSymbolScale", \@_, 4, $perlchartdir::PixelScale, [], $perlchartdir::PixelScale);
}
sub getLine
{
	return perlchartdir::method("LineLayer.getLine", \@_, 1, 0);
}

package ScatterLayer;
@ISA = ("LineLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}

package InterLineLayer;
@ISA = ("LineLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}

package SplineLayer;
@ISA = ("LineLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setTension
{
	perlchartdir::method("SplineLayer.setTension", \@_, 1);
}

package StepLineLayer;
@ISA = ("LineLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setAlignment
{
	perlchartdir::method("StepLineLayer.setAlignment", \@_, 1);
}		

	
#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to trendlayer.h
#///////////////////////////////////////////////////////////////////////////////////
package TrendLayer;
@ISA = ("Layer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setImageMapWidth
{
	perlchartdir::method("TrendLayer.setImageMapWidth", \@_, 1);
}
sub getLine
{
	return perlchartdir::method("TrendLayer.getLine", \@_, 0);
}
sub addConfidenceBand
{
	perlchartdir::method("TrendLayer.addConfidenceBand", \@_, 7, $perlchartdir::Transparent, 1, -1, -1, -1);
}
sub addPredictionBand
{
	perlchartdir::method("TrendLayer.addPredictionBand", \@_, 7, $perlchartdir::Transparent, 1, -1, -1, -1);
}
sub getSlope
{
	return perlchartdir::method("TrendLayer.getSlope", \@_, 0);
}
sub getIntercept
{
	return perlchartdir::method("TrendLayer.getIntercept", \@_, 0);
}
sub getCorrelation
{
	return perlchartdir::method("TrendLayer.getCorrelation", \@_, 0);
}
sub getStdError
{
	return perlchartdir::method("TrendLayer.getStdError", \@_, 0);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to arealayer.h
#///////////////////////////////////////////////////////////////////////////////////
package AreaLayer;
@ISA = ("Layer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setMinLabelSize
{
	perlchartdir::method("AreaLayer.setMinLabelSize", \@_, 1);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to hloclayer.h
#///////////////////////////////////////////////////////////////////////////////////
package HLOCLayer;
@ISA = ("Layer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setDataGap
{
	perlchartdir::method("HLOCLayer.setDataGap", \@_, 1);
}
sub setCandleStick
{
	perlchartdir::method("HLOCLayer.setCandleStick", \@_, 1);
}

package CandleStickLayer;
@ISA = ("HLOCLayer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}

package BoxWhiskerLayer;
@ISA = ("HLOCLayer");

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to xychart.h
#///////////////////////////////////////////////////////////////////////////////////
package PlotArea;
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub setBackground
{
	perlchartdir::method("PlotArea.setBackground", \@_, 3, -1, $perlchartdir::LineColor);
}
sub setBackground2
{
	perlchartdir::method("PlotArea.setBackground2", \@_, 2, $perlchartdir::Center);
}
sub setGridColor
{
	perlchartdir::method("PlotArea.setGridColor", \@_, 4, $perlchartdir::Transparent, -1, -1);
}
sub setGridWidth
{
	perlchartdir::method("PlotArea.setGridWidth", \@_, 4, -1, -1, -1);
}

package XYChart;
@ISA = ("BaseChart");


sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("XYChart.create", \@_, 5, 
		$perlchartdir::BackgroundColor, $perlchartdir::Transparent, 0);
	return bless($self, $class);
}
sub yAxis
{
	return new Axis(perlchartdir::method("XYChart.yAxis", \@_));	
}
sub yAxis2
{
	return new Axis(perlchartdir::method("XYChart.yAxis2", \@_));
}
sub syncYAxis
{
	perlchartdir::method("XYChart.syncYAxis", \@_, 2, 1, 0);
}
sub setYAxisOnRight
{
	perlchartdir::method("XYChart.setYAxisOnRight", \@_, 1, 1);
}
sub xAxis
{
	return new Axis(perlchartdir::method("XYChart.xAxis", \@_));	
}
sub xAxis2
{
	return new Axis(perlchartdir::method("XYChart.xAxis2", \@_));
}
sub setXAxisOnTop
{
	perlchartdir::method("XYChart.setXAxisOnTop", \@_, 1, 1);
}
sub swapXY
{
	perlchartdir::method("XYChart.swapXY", \@_, 1, 1);
}
sub setPlotArea
{
	return new PlotArea(perlchartdir::method("XYChart.setPlotArea", \@_, 9,	$perlchartdir::Transparent, 
		-1, $perlchartdir::LineColor, 0xc0c0c0, $perlchartdir::Transparent));
}
sub setClipping
{
	perlchartdir::method("XYChart.setClipping", \@_, 1, 0);
}
sub addBarLayer
{
	if (scalar(@_) == 1) { return addBarLayer2(@_); }
	return new BarLayer(perlchartdir::method("XYChart.addBarLayer", \@_, 4, -1, "", 0));
}
sub addBarLayer2
{
	return new BarLayer(perlchartdir::method("XYChart.addBarLayer2", \@_, 2, $Layer::Side, 0));
}
sub addBarLayer3()
{
	return new BarLayer(perlchartdir::method("XYChart.addBarLayer3", \@_, 4, [], [], 0));
}
sub addLineLayer
{
	if (scalar(@_) == 1) { return addLineLayer2(@_); }
	return new LineLayer(perlchartdir::method("XYChart.addLineLayer", \@_, 4, -1, "", 0));
}
sub addLineLayer2
{
	return new LineLayer(perlchartdir::method("XYChart.addLineLayer2", \@_, 2, $Layer::Overlay, 0));
}
sub addAreaLayer
{
	if (scalar(@_) == 1) { return addAreaLayer2(@_); }
	return new AreaLayer(perlchartdir::method("XYChart.addAreaLayer", \@_, 4, -1, "", 0));
}
sub addAreaLayer2
{
	return new AreaLayer(perlchartdir::method("XYChart.addAreaLayer2", \@_, 2, $Layer::Stack, 0));
}
sub addHLOCLayer
{
	if (scalar(@_) == 1) { return addHLOCLayer2(@_); }
	return new HLOCLayer(perlchartdir::method("XYChart.addHLOCLayer", \@_, 5, [], [], -1));
}
sub addHLOCLayer2
{
	return new HLOCLayer(perlchartdir::method("XYChart.addHLOCLayer2", \@_));
}
sub addScatterLayer
{
	return new ScatterLayer(perlchartdir::method("XYChart.addScatterLayer", \@_, 7, "", 
		$perlchartdir::SquareSymbol, 5, -1, -1));
}
sub addCandleStickLayer
{
	return new CandleStickLayer(perlchartdir::method("XYChart.addCandleStickLayer", \@_, 7, 
		0xffffff, 0x0, $perlchartdir::LineColor));
}
sub addBoxWhiskerLayer
{
	return new BoxWhiskerLayer(perlchartdir::method("XYChart.addBoxWhiskerLayer", \@_, 8, 
		[], [], [], -1, $perlchartdir::LineColor, $perlchartdir::LineColor));
}
sub addTrendLayer
{
	return new TrendLayer(perlchartdir::method("XYChart.addTrendLayer", \@_, 4, -1, "", 0));
}
sub addTrendLayer2
{
	return new TrendLayer(perlchartdir::method("XYChart.addTrendLayer2", \@_, 5, -1, "", 0));
}
sub addSplineLayer
{
	return new SplineLayer(perlchartdir::method("XYChart.addSplineLayer", \@_, 3, [], -1, ""));
}
sub addStepLineLayer
{
	return new StepLineLayer(perlchartdir::method("XYChart.addStepLineLayer", \@_, 3, [], -1, ""));
}
sub addInterLineLayer
{
	return new InterLineLayer(perlchartdir::method("XYChart.addInterLineLayer", \@_, 4, -1));
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to polarchart.h
#///////////////////////////////////////////////////////////////////////////////////
package PolarLayer;

%defaultArgs = (
	"setData"=>[undef, 3, -1, ""],
	"setSymbolScale"=>[undef, 2, $perlchartdir::PixelScale],
	"getImageCoor"=>[undef, 3, 0, 0],
	"getHTMLImageMap"=>[undef, 5, "", "", 0, 0],
	"setDataLabelStyle"=>["TextBox", 4, "", 8, $perlchartdir::TextColor, 0],
	"addCustomDataLabel"=>["TextBox", 6, "", 8, $perlchartdir::TextColor, 0]
	);

sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setData  { _r(@_); }
sub setAngles  { _r(@_); }
sub setBorderColor  { _r(@_); }
sub setLineWidth  { _r(@_); }

sub setDataSymbol
{
	if ((scalar(@_) < 3) && ($_[1]) && ($_[1] !~ m/^\d+$/)) {
		setDataSymbol2(@_);
	} else {
		perlchartdir::method("PolarLayer.setDataSymbol", \@_, 5, 7, -1, -1, 1);
	}
}
sub setDataSymbol2
{
	if (ref($_[1])) {
		setDataSymbol3(@_);
	} else {
		perlchartdir::method("PolarLayer.setDataSymbol2", \@_, 1);
	}
}
sub setDataSymbol3
{
	my $ret = eval {
		my ($self, $d) = @{perlchartdir::checkarg(\@_, 2)};
		perlchartdir::callMethod("PolarLayer.setDataSymbol3", $self->{"this"}, $d->{"this"});
		};
	if ((!$ret) && ($@)) { perlchartdir::printerr($@, 1); }
}
sub setSymbolScale  { _r(@_); }

sub getMaxR  { _r(@_); }
sub getMinR  { _r(@_); }

sub setImageMapWidth  { _r(@_); }
sub getImageCoor  { _r(@_); }
sub getHTMLImageMap  { _r(@_); }
sub setDataLabelFormat  { _r(@_); }
sub setDataLabelStyle  { _r(@_); }
sub addCustomDataLabel  { _r(@_); }


package PolarAreaLayer;
@ISA = ("PolarLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}

package PolarLineLayer;
@ISA = ("PolarLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setCloseLoop  { _r(@_); }
sub setGapColor
{
	perlchartdir::method("PolarLineLayer.setGapColor", \@_, 2, -1);
}

package PolarSplineLineLayer;
@ISA = ("PolarLineLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setTension  { _r(@_); }

package PolarSplineAreaLayer;
@ISA = ("PolarAreaLayer");
sub new
{
	my ($class, $this) = @_;
	my $self = {};
	$self->{"this"} = $this;
	return bless($self, $class);
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setTension  { _r(@_); }


package PolarChart;
@ISA = ("BaseChart");

%defaultArgs = (
	"setPlotArea"=>[undef, 6, $perlchartdir::Transparent, $perlchartdir::Transparent, 1],	
	"setGridColor"=>[undef, 4, $perlchartdir::LineColor, 1, $perlchartdir::LineColor, 1],
	"setGridStyle"=>[undef, 2, 1],
	"setStartAngle"=>[undef, 2, 1],
	"angularAxis"=>["AngularAxis", 0],
	"radialAxis"=>["Axis", 0],	
	"addAreaLayer"=>["PolarAreaLayer", 3, -1, ""],
	"addLineLayer"=>["PolarLineLayer", 3, -1, ""],
	"addSplineLineLayer"=>["PolarSplineLineLayer", 3, -1, ""],
	"addSplineAreaLayer"=>["PolarSplineAreaLayer", 3, -1, ""],
	);
sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("PolarChart.create", \@_, 5, 
		$perlchartdir::BackgroundColor, $perlchartdir::Transparent, 0);
	return bless($self, $class);
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setPlotArea  { _r(@_); }
sub setGridColor  { _r(@_); }
sub setGridStyle  { _r(@_); }
sub setStartAngle  { _r(@_); }
sub angularAxis  { _r(@_); }
sub radialAxis  { _r(@_); }
sub getXCoor  { _r(@_); }
sub getYCoor  { _r(@_); }
sub addExtraField  { _r(@_); }
sub addAreaLayer  { _r(@_); }
sub addLineLayer  { _r(@_); }
sub addSplineLineLayer  { _r(@_); }
sub addSplineAreaLayer  { _r(@_); }


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to chartdir.h
#///////////////////////////////////////////////////////////////////////////////////
package perlchartdir;

sub getCopyright
{
	return func("getCopyright", \@_);
}
sub getVersion
{
	return func("getVersion", \@_);
}
sub getDescription
{
	return func("getDescription", \@_)
}
sub getBootLog
{
	return func("getBootLog", \@_);
}
sub libgTTFTest
{
	return func("testFont", \@_, 5, "", 0, 8, 8, 0);
}

sub testFont 
{
	return func("testFont", \@_, 5, "", 0, 8, 8, 0);
}

sub setLicenseCode
{
	return func("setLicenseCode", \@_, 1);
}

sub chartTime
{
	if (scalar(@_) < 3) { return chartTime(@_); }
	return func("chartTime", \@_, 6, 0, 0, 0);
}

sub chartTime2
{
	return func("chartTime2", \@_, 1);
}


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to rantable.h
#///////////////////////////////////////////////////////////////////////////////////
package RanTable;

%defaultArgs = (
	"setCol2"=>[undef, 6, -1E+308, 1E+308],
	"setDateCol"=>[undef, 4, 0],
	"setHLOCCols"=>[undef, 6, 0, 1E+308]
	);

sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("RanTable.create", \@_, 3);
	return bless($self, $class);
}
sub DESTROY
{
	my $self = shift;
	perlchartdir::callMethod("RanTable.destroy", $self->{"this"})
}
sub _r
{	
	return perlchartdir::_r(\@_, \%defaultArgs);
}
sub setCol
{
	if (defined($_[4])) {
		setCol2(@_);
	} else {
		perlchartdir::method("RanTable.setCol", \@_, 3);
	}
}
sub setCol2 { _r(@_); }
sub setDateCol { _r(@_); }
sub setHLOCCols { _r(@_); }
sub getCol { _r(@_); }


#///////////////////////////////////////////////////////////////////////////////////
#//	bindings to datafilter.h
#///////////////////////////////////////////////////////////////////////////////////
package ArrayMath;

%defaultArgs = (
	"delta"=>[undef, 1, 1],
	"trim"=>[undef, 2, 0, -1],
	"insert"=>[undef, 2, -1],
	"insert2"=>[undef, 3, -1],
	"selectGTZ"=>[undef, 2, [], 0],
	"selectGEZ"=>[undef, 2, [], 0],
	"selectLTZ"=>[undef, 2, [], 0],
	"selectLEZ"=>[undef, 2, [], 0],
	"selectEQZ"=>[undef, 2, [], 0],
	"selectNEZ"=>[undef, 2, [], 0],
	"selectStartOfHour"=>[undef, 2, 1, 300],
	"selectStartOfDay"=>[undef, 2, 1, 3 * 3600],
	"selectStartOfWeek"=>[undef, 2, 1, 2 * 86400],
	"selectStartOfMonth"=>[undef, 2, 1, 5 * 86400],
	"selectStartOfYear"=>[undef, 2, 1, 60 * 86400],
	"movCorr"=>[undef, 2, []],
	"lowess"=>[undef, 2, 0.25, 0],
	"lowess2"=>[undef, 3, 0.25, 0]
	);
	
sub new 
{
	my $class = shift;
	my $self = {};
	$self->{"this"} = perlchartdir::func("ArrayMath.create", \@_, 1);
	return bless($self, $class);
}
sub DESTROY
{
	my $self = shift;
	perlchartdir::callMethod("ArrayMath.destroy", $self->{"this"})
}
sub _r
{	
	my $self = $_[0];
	my $ret = perlchartdir::_r(\@_, \%defaultArgs);
	if ($ret eq $self->{"this"}) {
		return $self;
	} else {
		return $ret;
	}
}
sub add
{
	my $self = $_[0];
	if (ref($_[1])) {
		perlchartdir::method("ArrayMath.add", \@_, 1);
		return $self;
	} else {
		return add2(@_);
	}
}
sub sub
{
	my $self = $_[0];
	if (ref($_[1])) {
		perlchartdir::method("ArrayMath.sub", \@_, 1);
		return $self;
	} else {
		return sub2(@_);
	}
}
sub mul
{
	my $self = $_[0];
	if (ref($_[1])) {
		perlchartdir::method("ArrayMath.mul", \@_, 1);
		return $self;
	} else {
		return mul2(@_);
	}
}
sub div
{
	my $self = $_[0];
	if (ref($_[1])) {
		perlchartdir::method("ArrayMath.div", \@_, 1);
		return $self;
	} else {
		return div2(@_);
	}
}
sub add2 { _r(@_); }
sub sub2 { _r(@_); }
sub mul2 { _r(@_); }
sub div2 { _r(@_); }
sub delta { _r(@_); }
sub abs { _r(@_); }
sub acc { _r(@_); }
sub trim { _r(@_); }
sub insert { _r(@_); }
sub insert2 { _r(@_); }
sub replace { _r(@_); }
sub selectGTZ { _r(@_); }
sub selectGEZ { _r(@_); }
sub selectLTZ { _r(@_); }
sub selectLEZ { _r(@_); }
sub selectEQZ { _r(@_); }
sub selectNEZ { _r(@_); }
sub selectStartOfHour { _r(@_); }
sub selectStartOfDay { _r(@_); }
sub selectStartOfWeek { _r(@_); }
sub selectStartOfMonth { _r(@_); }
sub selectStartOfYear { _r(@_); }
sub movAvg { _r(@_); }
sub expAvg { _r(@_); }
sub movMed { _r(@_); }
sub movPercentile { _r(@_); }
sub movMax { _r(@_); }
sub movMin { _r(@_); }
sub movStdDev { _r(@_); }
sub movCorr { _r(@_); }
sub lowess { _r(@_); }
sub lowess2 { _r(@_); }
sub result { _r(@_); }
sub max { _r(@_); }
sub min { _r(@_); }
sub maxIndex { _r(@_); }
sub minIndex { _r(@_); }
sub avg { _r(@_); }
sub sum { _r(@_); }
sub med { _r(@_); }
sub percentile { _r(@_); }


#///////////////////////////////////////////////////////////////////////////////////
#//	Utility functions
#///////////////////////////////////////////////////////////////////////////////////
package perlchartdir;

#
#Utility function to create a unique temporary file name. Also 
#automatically removes temporary files that are older than 3 minutes.
#
sub tmpFile2 
{
	my ($path, $lifeTime) = @_;

	#remove trailing slashes
	$path =~ s/[\\\/]*$//;

	#
	#remove old temporary files
	#
	if (!opendir(DIR, $path)) {
		#make the directory from the root up in case it does not exist
		my @acc;
		for (split(/[\\\/]/, $path)) {
			push @acc, $_;
			mkdir(join("/", @acc), 0777);
		}
		opendir(DIR, $path) or return "$path/cannot_open_directory";	
	}
	unlink(map { abs(time - (stat("$path/$_"))[9]) > $lifeTime ? "$path/$_" : () } readdir(DIR));
	closedir(DIR);
	
	#
	#get unique file name. use Apache mod_perl environmental variable "UNIQUE_ID" if available, 
	#otherwise generate it ourselves.
	#
	return $ENV{UNIQUE_ID} if exists $ENV{UNIQUE_ID};
	return $ENV{REMOTE_ADDR}.$ENV{REMOTE_PORT}.$$.time;
}

sub tmpFile
{
	#for compatibility with ChartDirector Ver 2.5
	my $path = (shift or "/tmp/tmp_charts");
	my $lifeTime = (shift or 600);
	return $path . "/" . tmpFile2($path, $lifeTime);
}

1;
