#!/usr/bin/perl
use DBI;

#standard code to parse HTTP query parameters
my %query = map {my($k,$v) = split(/=/)} split(/&/, $ENV{"QUERY_STRING"});

#
#Perform the database query to get the required data. The selected year
#should be passed in as a query parameter called "year"
#
my $selectedYear = ($query{"year"} or 2001);

#
#Create an SQL statement to get the revenues of each month for the
#selected year.
#
my $SQLstatement = "Select Month(TimeStamp) - 1, " .
      "Software, Hardware, Services " .
      "From revenue Where Year(TimeStamp)=$selectedYear";

#
#Read in the revenue data into arrays
#
my $dbh = DBI->connect('dbi:mysql:sample','test','test');
my $sth = $dbh->prepare($SQLstatement);

my @software = (0) x 12;
my @hardware = (0) x 12;
my @services = (0) x 12;
$sth->execute();
while (my @row = $sth->fetchrow_array) {
	$software[$row[0]] = $row[1];
	$hardware[$row[0]] = $row[2];
	$services[$row[0]] = $row[3];
}
$dbh->disconnect;

#Serialize the data into a string to be used as HTTP query parameters
my $httpParam = "year=$selectedYear&software=".join(",", @software) .
   "&hardware=".join(",", @hardware) . "&services=".join(",", @services);

print "Content-type: text/html\n\n";
print <<EndOfHTML
<html>
<body>
<h1>Database Integration Demo (2)</h1>
<p style="width:500px;">This example demonstrates creating a chart using 
data from a database. The database query is performed in the containing
HTML page. The data are then passed to the chart generation pages as 
HTTP GET parameters.</p>

<ul>
	<li><a href="viewsource.pl?file=$ENV{"SCRIPT_NAME"}">
		View containing HTML page source code
	</a></li>
	<li><a href="viewsource.pl?file=dbdemo2a.pl">
		View chart generation page source code for upper chart
	</a></li>
	<li><a href="viewsource.pl?file=dbdemo2b.pl">
		View chart generation page source code for lower chart
	</a></li>
</ul>

<form action="$ENV{"SCRIPT_NAME"}">
	I want to obtain the revenue data for the year 
	<select name="year">
		<option value="1990">1990
		<option value="1991">1991
		<option value="1992">1992
		<option value="1993">1993
		<option value="1994">1994
		<option value="1995">1995
		<option value="1996">1996
		<option value="1997">1997
		<option value="1998">1998
		<option value="1999">1999
		<option value="2000">2000
		<option value="2001">2001
	</select>
	<input type="submit" value="OK">
</form>

<SCRIPT>
	//make sure the select box displays the current selected year.
	document.forms[0].year.selectedIndex = ${\($selectedYear - 1990)};
</SCRIPT>

<img src="dbdemo2a.pl?$httpParam"><br>
<img src="dbdemo2b.pl?$httpParam">

</body>
</html>
EndOfHTML
;
