#!/usr/bin/perl

#Include current script directory in the module path (needed on Microsoft IIS).
#This allows this script to work by copying ChartDirector to the same directory
#as the script (as an alternative to installation in Perl module directory)
use File::Basename;
use lib dirname($0);

use perlchartdir;

#The data for the chart
my $dataY0 = [4, 4.5, 5, 5.25, 5.75, 5.25, 5, 4.5, 4, 3, 2.5, 2.5];
my $dataX0 = [perlchartdir::chartTime(1997, 1, 1), perlchartdir::chartTime(1998,
    6, 25), perlchartdir::chartTime(1999, 9, 6), perlchartdir::chartTime(2000,
    2, 6), perlchartdir::chartTime(2000, 9, 21), perlchartdir::chartTime(2001,
    3, 4), perlchartdir::chartTime(2001, 6, 8), perlchartdir::chartTime(2002, 2,
    4), perlchartdir::chartTime(2002, 5, 19), perlchartdir::chartTime(2002, 8,
    16), perlchartdir::chartTime(2002, 12, 1), perlchartdir::chartTime(2003, 1,
    1)];

my $dataY1 = [7, 6.5, 6, 5, 6.5, 7, 6, 5.5, 5, 4, 3.5, 3.5];
my $dataX1 = [perlchartdir::chartTime(1997, 1, 1), perlchartdir::chartTime(1997,
    7, 1), perlchartdir::chartTime(1997, 12, 1), perlchartdir::chartTime(1999,
    1, 15), perlchartdir::chartTime(1999, 6, 9), perlchartdir::chartTime(2000,
    3, 3), perlchartdir::chartTime(2000, 8, 13), perlchartdir::chartTime(2001,
    5, 5), perlchartdir::chartTime(2001, 9, 16), perlchartdir::chartTime(2002,
    3, 16), perlchartdir::chartTime(2002, 6, 1), perlchartdir::chartTime(2003,
    1, 1)];

#Create a XYChart object of size 500 x 270 pixels, with a pale blue (0xe0e0ff)
#background, a light blue (0xccccff) border, and 1 pixel 3D border effect.
my $c = new XYChart(500, 270, 0xe0e0ff, 0xccccff, 1);

#Set the plotarea at (50, 50) and of size 420 x 180 pixels, using white
#(0xffffff) as the plot area background color. Turn on both horizontal and
#vertical grid lines with light grey color (0xc0c0c0)
$c->setPlotArea(50, 50, 420, 180, 0xffffff)->setGridColor(0xc0c0c0, 0xc0c0c0);

#Add a legend box at (55, 25) (top of the chart) with horizontal layout. Use 10
#pts Arial Bold Italic font. Set the background and border color to Transparent.
$c->addLegend(55, 25, 0, "arialbi.ttf", 10)->setBackground(
    $perlchartdir::Transparent);

#Add a title to the chart using 14 points Times Bold Itatic font, using blue
#(0x9999ff) as the background color
$c->addTitle("Interest Rates", "timesbi.ttf", 14)->setBackground(0x9999ff);

#Set the y axis label format to display a percentage sign
$c->yAxis()->setLabelFormat("{value}%");

#Add a red (0xff0000) step line layer to the chart and set the line width to 2
#pixels
my $layer0 = $c->addStepLineLayer($dataY0, 0xff0000, "Country AAA");
$layer0->setXData($dataX0);
$layer0->setLineWidth(2);

#Add a blue (0x0000ff) step line layer to the chart and set the line width to 2
#pixels
my $layer1 = $c->addStepLineLayer($dataY1, 0xff, "Country BBB");
$layer1->setXData($dataX1);
$layer1->setLineWidth(2);

#output the chart
binmode(STDOUT);
print "Content-type: image/png\n\n";
print $c->makeChart2($perlchartdir::PNG);
